==> scanning-plugin-0.15.0-sources.jar: com/estimote/scanning_plugin/packet_provider/service/PacketProviderWrapperServiceHelper.kt <==
package com.estimote.scanning_plugin.packet_provider.service

import android.app.Notification
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Binder
import android.os.Build
import android.os.IBinder
import com.estimote.scanning_plugin.packet_provider.PacketProvider
import io.reactivex.Observable
import io.reactivex.ObservableEmitter

internal class PacketProviderWrapperServiceHelper {
    private val NOTIFICATION_INTENT_KEY = "NOTIFICATION"

    data class LocalBinder(val service: PacketProviderWrapperService) : Binder()

    fun startAndBind(applicationContext: Context, notification: Notification): Observable<PacketProvider> =
            Observable.create<PacketProvider> { emitter ->
                startServiceWithNotification(applicationContext, notification)
                val connection: ServiceConnection = createServiceConnection(emitter)
                emitter.setCancellable { applicationContext.unbindService(connection) }
                applicationContext.bindService(Intent(applicationContext, PacketProviderWrapperService::class.java), connection, Context.BIND_AUTO_CREATE)
            }

    fun extractNotification(intent: Intent?) =
            intent?.extras?.getParcelable<Notification>(NOTIFICATION_INTENT_KEY)


    private fun createServiceConnection(emitter: ObservableEmitter<PacketProvider>): ServiceConnection =
            object : ServiceConnection {
                override fun onServiceConnected(name: ComponentName, binder: IBinder) = emitter.onNext((binder as LocalBinder).service)
                override fun onServiceDisconnected(name: ComponentName?) = Unit
            }

    private fun startServiceWithNotification(applicationContext: Context, notification: Notification) {
        val intent = Intent(applicationContext, PacketProviderWrapperService::class.java).putExtra(NOTIFICATION_INTENT_KEY, notification)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) applicationContext.startForegroundService(intent)
        else applicationContext.startService(intent)
    }
}

==> scanning-plugin-0.18.0-sources.jar: com/estimote/scanning_plugin/packet_provider/service/PacketProviderWrapperServiceHelper.kt <==
package com.estimote.scanning_plugin.packet_provider.service

import android.app.Notification
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Binder
import android.os.Build
import android.os.IBinder
import com.estimote.scanning_plugin.packet_provider.PacketProvider
import com.wafel.skald.api.createLogger
import io.reactivex.Observable
import io.reactivex.ObservableEmitter

internal class PacketProviderWrapperServiceHelper(private val applicationContext: Context,
                                                  private val notification: Notification) {
    private val NOTIFICATION_INTENT_KEY = "NOTIFICATION"
    private val logger = createLogger(PacketProviderWrapperServiceHelper::class.java)

    data class LocalBinder(val service: PacketProviderWrapperService) : Binder()

    private val packetProviderLoader = Observable.create<PacketProvider> { emitter ->
        startServiceWithNotification(applicationContext, notification)
        val connection: ServiceConnection = createServiceConnection(emitter)
        emitter.setCancellable {
            logger.debug("Unbinding from the foreground service.")
            applicationContext.unbindService(connection)
        }
        applicationContext.bindService(Intent(applicationContext, PacketProviderWrapperService::class.java), connection, Context.BIND_AUTO_CREATE)
    }.replay().refCount()

    fun startAndBind(): Observable<PacketProvider> = packetProviderLoader

    fun extractNotification(intent: Intent?) =
            intent?.extras?.getParcelable<Notification>(NOTIFICATION_INTENT_KEY)


    private fun createServiceConnection(emitter: ObservableEmitter<PacketProvider>): ServiceConnection =
            object : ServiceConnection {
                override fun onServiceConnected(name: ComponentName, binder: IBinder) = emitter.onNext((binder as LocalBinder).service)
                override fun onServiceDisconnected(name: ComponentName?) = Unit
            }

    private fun startServiceWithNotification(applicationContext: Context, notification: Notification) {
        val intent = Intent(applicationContext, PacketProviderWrapperService::class.java).putExtra(NOTIFICATION_INTENT_KEY, notification)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) applicationContext.startForegroundService(intent)
        else applicationContext.startService(intent)
    }
}
